!!$=head1 NAME
!!$
!!$NHelicon - Non-stationary helix convolution and deconvolution 
!!$
!!$=head1 SYNOPSIS
!!$
!!$Helicon < in.H filt= adj=0 div=0 > out.H
!!$
!!$=head1 DESCRIPTION
!!$
!!$Applies helix convolution (polynomial multiplication) or deconvolution
!!$(polynomial division). One is the exact inverse of the other. Watch
!!$for helical boundary conditions.
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from history
!!$
!!$B<integer array> I<n> - reads I<n1>, I<n2>, I<n3>, ... 
!!$
!!$=item from par
!!$
!!$B<filename> I<filt> - helix filter file
!!$
!!$B<integer> I<adj>=0 - apply adjoint (backward) filtering
!!$
!!$B<integer> I<div>=0 - apply inverse recursive filtering (polynomial division)
!!$
!!$=item from aux (filt)
!!$
!!$B<integer array> I<h> - helix grid (can be I<h1>, I<h2>, ...)
!!$
!!$B<integer array> I<lag>=1,...,n1 - comma separated list of filter lags
!!$
!!$B<real> I<a0>=1 - zero-lag filter coefficient
!!$
!!$=back
!!$
!!$=head1 EXAMPLES
!!$
!!$F<gee/pch/tvdecon90/>
!!$
!!$=head1 MODULES
!!$
!!$F<nhelicon.lop>, F<npolydiv.lop>, F<nhelix.r90>, F<helix.r90>, F<regrid.r90>
!!$
!!$=head1 SEA ALSO
!!$
!!$L<Missfill>, L<Pef>
!!$
!!$=cut
!%
program NHelico
  use sep
  use nhelix
  use nhelicon
  use npolydiv
  use helix
  use regrid

  implicit none
  integer                             :: ia, na, nx, stat, dim, ix, n123
  real                                :: a0
  logical                             :: adj, add = .false., div
  real,    dimension (:), allocatable :: pp, qq, pnh
  integer, dimension (:), allocatable :: n, m, pch, nh, lag
  type( nfilter)                      :: aa

  call sep_init (SOURCE)
  dim = sep_dimension()
  allocate (n (dim), m(dim))
  call from_history (n)
  n123 = product (n)

  if (exist_file ("nh")) then
     call from_aux ("nh", nx)
  else
     call from_aux ("filt", na, nx)
  end if

  allocate (pp (n123), qq (n123))

  allocate (pch (n123), nh (nx))
  if (exist_file ("pch")) then     
     call sep_read (pp, "pch")
     pch = pp
  else
     if (nx /= n123) call erexit ("wrong dimensions!")
     pch = (/ (ix, ix = 1, nx) /)
  end if

  if (exist_file ("nh")) then
     allocate (pnh (nx))
     call sep_read (pnh, "nh")
     nh = pnh
     deallocate (pnh)
  else
     nh = na
  end if
  call nallocate (aa, nh, pch)
  deallocate (pch)

  call from_aux ("filt", "a0", a0, 1.)

  if (exist_file ("lag")) then
     do ix = 1, nx
        na = nh(ix)
        call sep_read (qq(:na), "lag")
        aa%hlx(ix)%lag = qq(:na)
     end do
  else
     allocate (lag (na))
     call from_aux ("filt", "lag", lag, (/ (ia, ia = 1, na) /))
     do ix = 1, nx
        aa%hlx(ix)%lag = lag
     end do
  end if
  deallocate (nh)
    
  call from_aux ("filt", "n",m,n)
  do ix = 1, nx
     call regridn (m,n, aa%hlx(ix))
  end do

  call from_par ("adj",adj,.false.)
  call from_par ("div",div,.false.)
  call sep_close ()

  if (adj) then
     call sep_read (qq)
  else
     call sep_read (pp)
  end if

  do ix = 1, nx
     call sep_read (aa%hlx(ix)%flt, "filt")
     aa%hlx(ix)%flt = aa%hlx(ix)%flt/a0
  end do

  if (div) then
     call npolydiv_init (n123, aa)
     stat = npolydiv_lop (adj,add,pp,qq)
     call npolydiv_close()
  else
     call nhelicon_init (aa)
     stat = nhelicon_lop (adj,add,pp,qq)
  end if

  if (adj) then
     call sep_write (pp)
  else
     call sep_write (qq)
  end if

  deallocate (n, m, pp, qq)
  call ndeallocate (aa)
  call exit (0)
end program NHelico


