!!$=head1 NAME
!!$
!!$NHelicon - Non-stationary helix convolution and deconvolution 
!!$
!!$=head1 SYNOPSIS
!!$
!!$Helicon < in.H filt= adj=0 div=0 > out.H
!!$
!!$=head1 DESCRIPTION
!!$
!!$Applies helix convolution (polynomial multiplication) or deconvolution
!!$(polynomial division). One is the exact inverse of the other. Watch
!!$for helical boundary conditions.
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from history
!!$
!!$B<integer array> I<n> - reads I<n1>, I<n2>, I<n3>, ... 
!!$
!!$=item from par
!!$
!!$B<filename> I<filt> - helix filter file
!!$
!!$B<integer> I<adj>=0 - apply adjoint (backward) filtering
!!$
!!$B<integer> I<div>=0 - apply inverse recursive filtering (polynomial division)
!!$
!!$=item from aux (filt)
!!$
!!$B<integer array> I<h> - helix grid (can be I<h1>, I<h2>, ...)
!!$
!!$B<integer array> I<lag>=1,...,n1 - comma separated list of filter lags
!!$
!!$B<real> I<a0>=1 - zero-lag filter coefficient
!!$
!!$=back
!!$
!!$=head1 EXAMPLES
!!$
!!$F<gee/pch/tvdecon90/>
!!$
!!$=head1 MODULES
!!$
!!$F<nhelicon.lop>, F<npolydiv.lop>, F<nhelix.r90>, F<helix.r90>, F<regrid.r90>
!!$
!!$=head1 SEA ALSO
!!$
!!$L<Missfill>, L<Pef>
!!$
!!$=cut
!%
program NHelico
  use sep
  use io_nfilt
  use nhelix
  use nhelicon
  use npolydiv
  use helix
  use regrid
  use error_check

  implicit none
  integer                             :: ia, na, nx, stat, dim, ix, n123, readstat
  real                                :: a0
  logical                             :: adj, add = .false., div
  real,    dimension (:), allocatable :: pp, qq, pnh, data
  integer, dimension (:), allocatable :: n, m, pch, nh, lag
  type( nfilter)                      :: aa
  character(len=1024)                 :: filt

  call sep_init (SOURCE)
  dim = sep_dimension()
  allocate (n (dim), m(dim))
  call from_history (n)
  n123 = product (n)
  allocate(pp(n123),qq(n123))

  call from_param ("adj",adj,.false.)
  call from_param ("div",div,.false.)
  call from_param ("filt",filt,'filt.H')
  readstat = read_nfilt(filt,aa)
  nx=size(aa%hlx)

!  do ix = 1, nx
!     call regridn (m,n, aa%hlx(ix))
!  end do
  call sep_close ()

  if (adj) then
     call sep_read (qq)
  else
     call sep_read (pp)
  end if

call pefcheck(aa)
stat=0
  do ix=1,maxval(aa%pch)
     if (maxval(aa%hlx(ix)%flt)>stat) then
        stat=maxval(aa%hlx(ix)%flt)
     end if
  end do
write(0,*)"pef coeffs",stat
call rmms(pp,"   pp")
call rmms(qq,"   qq")
  if (div) then
     call npolydiv_init (n123, aa)
     stat = npolydiv_lop (adj,add,pp,qq)
     call npolydiv_close()
  else
     call nhelicon_init (aa)
     stat = nhelicon_lop (adj,add,pp,qq)
  end if
call rmms(pp,"   pp")
call rmms(qq,"   qq")

  if (adj) then
     call sep_write (pp)
  else
     call sep_write (qq)
  end if

  deallocate (n, m, pp, qq)
  call ndeallocate (aa)
  call exit (0)
end program NHelico


