!!$=head1 NAME
!!$
!!$NPef - Estimate Non-stationary PEF in N dimensions
!!$
!!$=head1 SYNOPSIS
!!$
!!$Pef < data.H a= center=1 gap=0 [maskin=] [maskout=] > pef.H
!!$
!!$=head1 DESCRIPTION
!!$
!!$Estimates PEF by least squares, using helix convolution. 
!!$Can ignore missing data 
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from history
!!$
!!$B<integer array> I<n> - reads I<n1>, I<n2>, I<n3>, etc.
!!$
!!$=item from par
!!$
!!$B<integer> I<niter>=100 - number of iterations
!!$
!!$B<real> I<epsilon>=0.01 - regularization parameter
!!$ 
!!$B<integer array> I<a>= - filter size
!!$
!!$B<integer array> I<center>=1 - zero-lag position (filter centering)
!!$
!!$B<integer array> I<gap>=0 - filter gap
!!$
!!$B<filename> I<maskin, maskout> (optional) - data selectors 
!!$
!!$=item to history
!!$
!!$B<integer array> I<lag> - comma separated list of filter lags
!!$
!!$=back
!!$
!!$=head1 MODULES
!!$
!!$F<nhelix.r90>, F<createnhelixmod.r90>, F<nmisinput.r90>, F<npef.r90>, 
!!$
!!$=head1 SEE ALSO
!!$
!!$L<MSPef>, L<Pef>, L<NHelicon>
!!$
!!$=cut
!%
program NPef3
  use sep 
  use nhelix
  use createnhelixmod
  use nmisinput
  use npef
 
  implicit none
  integer, dimension (:), allocatable :: n, a, center, gap, lag, pch, nh
  integer                             :: dim, n123, nf, i, niter, nbf, nbp
  integer                             :: id, ip, if, np
  real,    dimension (:), pointer     :: dd, kk
  real,    dimension (:), allocatable :: pp
  type( nfilter)                      :: aa, bb   
  character (len = 6)                 :: varname
  real                                :: eps

  call sep_init (SOURCE)
  dim = sep_dimension ()
  allocate (n (dim), center (dim), a (dim), gap (dim))
  call from_history (n)
  call to_history ("n",n)
  call from_par ("a", a)
  center = 1
  where (a (2:) > 1) center (:dim-1) = a(:dim-1)/2 + 1
  call from_par ("center", center, center)
  gap = 0
  call from_par ("gap", gap, gap)
 
  n123 = product (n)
  allocate (dd (n123), kk (n123)) 
  call sep_read (dd)
  dd = dd / maxval (abs (dd))
  call random_number (kk)
  dd = dd + 100.*epsilon(kk)*(kk-0.5)

  if (exist_file ("maskin")) then
     call sep_read (kk, "maskin")
  else
     kk = 1.
  end if

  allocate (pp (n123)) 
  if (exist_file ("pch")) then
     call sep_read (pp, "pch")
     np = maxval (pp)
  else
     np = n123
     pp = (/ (ip, ip = 1, np) /)
  end if

  aa = createnhelix( n, center, gap, a, int(pp))         ! allocate PEF
  deallocate (pp)

  nf = size (aa%hlx(1)%lag)
  call find_mask( (kk /= 0.), aa)   ! missing data

  call from_par ("niter", niter, 100)
  call from_par ("epsilon", eps, 0.01)

  call to_history ("n1", nf)
  call to_history ("n2", np)

  do i=3, dim
     call to_history (axisname(i),1)
  end do
  do i=1, nf
     if (i < 10) then
        write (varname,"(a,i1)") "lag",i
     else if (i < 100) then
        write (varname,"(a,i2)") "lag",i
     else
        write (varname,"(a,i3)") "lag",i
     end if
     call to_history_int (varname, aa%hlx(1)%lag (i))
  end do

  if (exist_file ("maskout")) then
     where (aa%mis) 
        kk = 0.
     elsewhere
        kk = 1.
     end where
     do i=1, dim
        call to_history (axisname(i),n(i),"maskout")
     end do
     call to_history ("esize",4,"maskout")
     call sep_write (kk, "maskout")
  end if

!  call from_aux ("filt", nbf, nbp)
  nbf=nf
  nbp=np
  if (exist_file ("filt_pch")) then
     allocate (pp (np))
     call sep_read (pp, "filt_pch")
  else
     if (nbp /= np) call erexit ("Wrong filter size!")
  end if

  allocate (pch (nf*np), nh (nbp)); nh = nbf
  id = 0
  do if = 1, nf
     do ip = 1, np
        id = id + 1
        if (allocated (pp)) then
           pch (id) = pp (ip)
        else
           pch (id) = ip
        end if
     end do
  end do
  if (allocated (pp)) deallocate (pp)

  call nallocate (bb, nh, pch)
  deallocate (pch, nh)

  if (exist_file ("filt_lag")) then
     do ip = 1, nbp
        call sep_read (kk(:nbf),"filt_lag")
        bb%hlx(ip)%lag = kk(:nbf)*nf
     end do
  else
     allocate (lag (nbf))     
!     call from_aux ("filt","lag", lag)
     lag=aa%hlx(1)%lag
     do ip = 1, nbp
        bb%hlx(ip)%lag = lag*nf
     end do
     deallocate (lag)
  end if

  do ip = 1, nbp
     !call sep_read (bb%hlx(ip)%flt, "filt")  
     bb%hlx(ip)%flt=0
  end do
  call sep_close ()  

  call find_pef ( dd, aa, bb, niter, eps, nf)   ! estimate aa

  do ip = 1, np
     call sep_write (aa%hlx(ip)%flt)
  end do

  deallocate (dd, kk)
  deallocate (n, center, a, gap)
  call ndeallocate( aa)
  call ndeallocate( bb)
  call exit (0)
end program NPef3

 
